<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;

use Illuminate\Support\Str;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;

use App\Models\Page;
use App\Models\User;
use App\Actions\Fortify\CreateNewUser;
use App\Mail\SendPortalRegistrationEmailAll;


class CreateUsersFromEventRegistrations extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'users:event-registrations {slug} {--debug=true} {--status=2}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        $page = Page::where('slug', $this->argument('slug'))->first();

        if (!$page) {
            $this->error('Page not found');
            return Command::FAILURE;
        }

        // Get registrations from page
        // Status 2 = Paid, Status 1 = Free of Charge, Status 0 = Unpaid
        $registrations = $page->registrations()->where('status', $this->option('status'))->get();

        if (!$registrations) {
            $this->error('No registrations found');
            return Command::FAILURE;
        }

        $emails = [
            'joao.lima@thetechcapital.com',
            'rob@signalfire.co.uk'
        ];

        // Iterate each registration
        foreach($registrations as $registration)
        {
            $attendees = $registration->attendees()->get();

            if (!$attendees) {
                $this->error('No attendees found for registration');
                return Command::FAILURE;
            }

            foreach($attendees as $attendee) {
                $user = User::where('email', $attendee->email)->first();

                try {
                    // Create a password
                    $password = Str::random(12);

                    // If the user doesnt exist then create a new user
                    if (!$user) {

                        if ($this->option('debug') === 'true') {
                            $this->info('DEBUG : User with email ' . $attendee->email . ' not found, creating');
                        } else {
                            $this->info('User with email ' . $attendee->email . ' not found, creating');

                            $user = (new CreateNewUser)->create([
                                'forename' => $attendee->forename,
                                'surname' => $attendee->surname,
                                'email' => $attendee->email,
                                'password' => $password,
                                'password_confirmation' => $password,
                                'portal' => false
                            ]);

                            $emails[] = $attendee->email;
                        }

                    } else {

                        if ($this->option('debug') === 'true') {
                            $this->info('DEBUG : User with email ' . $attendee->email . ' found, changing password');
                        } else {
                            if (!in_array($attendee->email, $emails)) {
                                $this->info('User with email ' . $attendee->email . ' found, changing password');
                                $user->password = Hash::make($password);
                                $user->portal = false;
                                $user->save();
                            } else {
                                $this->info('User with email ' . $attendee->email . 'found but already processed, skipping');
                            }
                        }
                    }

                    // If the user was created or already exists then send the portal registration email
                    if ($user)
                    {
                        if ($this->option('debug') === 'true') {
                            $this->info('DEBUG : Portal registration email sent to ' . $attendee->email);
                        } else {
                            $this->info('Portal registration email sent to ' . $attendee->email);
                            if (!in_array($attendee->email, $emails)) {              
                                Mail::to($attendee->email)
                                    ->send(new SendPortalRegistrationEmailAll($registration, $user, $password));
                            }
                        }

                    }

                    // Sleep for 5 second to avoid rate limiting
                    sleep(5);
                } catch (Exception $ex) {
                    $this->error('Error creating user / sending mail: ' . $ex->getMessage());
                }
            }
        }

        return Command::SUCCESS;
    }
}
