<?php

namespace App\Http\Livewire\Forms;

use Livewire\Component;

use Illuminate\Http\Request;
use Illuminate\Mail\Mailable;

use App\Mail\SendContactEmail;
use App\Mail\SendRegistrationEmail;
use App\Mail\SendFreeRegistrationEmail;
use App\Models\Form;
use App\Models\Registration;
use App\Models\PromoCode;

use App\Actions\Forms\GenerateValidationRules;
use App\Actions\Forms\SendFormEmail;
use App\Actions\Forms\ResolveRegistrationData;
use App\Actions\Forms\ResolveModelProperties;
use App\Actions\Forms\CheckForFreeRegistration;
use App\Actions\Forms\ResolveRelationsData;
use App\Actions\Forms\RequiredModelFieldsValidator;

use Mail;
use Schema;
use Log;

class DynamicForm extends Component
{
    public $page;
    public $component;
    public $form;
    public $fields;
    public $data = [];

    protected $listeners = [
        'updateData',
        'submitForm',
    ];

    private function validationRules(): array
    {
        return (new GenerateValidationRules)->execute($this->fields);
    }

    private function sendEmail(Mailable $email): bool
    {
        return (new SendFormEmail)->execute($this->form, $email);
    }

    private function sendEmailContact($data): bool
    {
        return $this->sendEmail(new SendContactEmail($this->form, $data), [], [config('mail.default_to.address')]);
    }

    private function sendEmailRegistration($data): bool
    {
        return $this->sendEmail(new SendRegistrationEmail($this->form, $data), [], [config('mail.default_to.address')]);
    }    

    private function sendCustomerFreeRegistrationEmail($registration): bool
    {
        return $this->sendEmail(new SendFreeRegistrationEmail($this->form, $registration), [$registration->email], [config('mail.default_to.address')]);
    }

    private function resolveRelationsData(): array
    {
        return (new ResolveRelationsData)->execute($this->fields, $this->data);
    }

    private function getModelProperties(): array
    {
        return (new ResolveModelProperties)->execute(Registration::class, ['id', 'created_at', 'updated_at']);        
    }

    private function resolveRegistrationData(): array
    {
        return (new ResolveRegistrationData)->execute($this->page, session()->get('price'), $this->getModelProperties(), $this->data);
    }

    private function validateHasRequiredRegistrationModelFields(): void
    {
        (new RequiredModelFieldsValidator)->execute($this->data, [ 'title', 'forename', 'surname', 'email' ]);
    }

    private function setRegistrationSessionData($registration): void
    {
        // Save registration in session
        session()->put('registration', $registration);
        // Save first attendee to session
        session()->put('attendees', [
            [
                'forename' => $registration->forename,
                'surname' => $registration->surname,
                'email' => $registration->email,
            ]
        ]);                               
    }

    public function mount($page, $component)
    {
        $this->page = $page;
        $this->component = $component;

        $this->form = Form::where('id', $component['form'])->first();

        if ($this->form) {
            $this->fields = json_decode($this->form->fields, true);
        }
    }

    public function updateData($key, $value): void
    {
        $this->data[$key] = $value;
    }

    public function submit()
    {
        //Get validation rules
        $rules = $this->validationRules();

        // Validate rules
        if ($rules) {
            $this->validate($rules);
        }

        // At this point use $this->form->type to decide what happens to the data next
        switch(strtolower($this->form->type)) {
            case 'email':
                // Resolve the related models data
                $data = $this->resolveRelationsData();
                // If data is resolved, send email and redirect
                if ($data) {
                    $this->sendEmailContact($data);
                    return redirect($this->form->url);
                }
                break;
            case 'registration':
                // Validate that data has the required fields for the registration model
                $this->validateHasRequiredRegistrationModelFields();
                // Resolve the registration data putting anyting else in the extra field
                $data = $this->resolveRegistrationData();
                // If data is resolved, create the registration and send email and redirect
                if ($data) {
                    // Create the registration
                    $registration = Registration::create($data);
                    // If registration is created, send email and redirect and save registration in session
                    if ($registration) { 
                        // Send email registration
                        $this->sendEmailRegistration($registration);
                        // Save registration in session
                        $this->setRegistrationSessionData($registration);
                        // Check if the registration has a promo code provided
                        if ($registration->promo) {
                            $free = (new CheckForFreeRegistration)->execute($registration);                            
                            if ($free) {
                                // Send free registration email
                                $this->sendCustomerFreeRegistrationEmail($registration);
                                // Redirect to the skip checkout page
                                return redirect()->route('page.checkout.skip', ['page' => $this->page->slug]);
                            }
                        }
                        // Redirect to the form URL
                        return redirect($this->form->url);
                    }
                }
                // If data is not resolved, and/or registration not created redirect to the error page
                return redirect()->route('page.error', ['page' => $this->page->slug]);
                break;
            default:
                return redirect()->route('page.error', ['page' => $this->page->slug]);
        }
    }

    public function render(): \Illuminate\View\View
    {
        return view('livewire.forms.dynamic-form');
    }
}
