<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Barryvdh\DomPDF\Facade\Pdf;

use App\Models\Registration;
use App\Actions\Ecommerce\PriceCalculator;

use App\Mail\SendRegistrationPaidClientEmail;
use App\Mail\SendRegistrationPaidAdminEmail;

use DB;
use Log;
use Mail;

class ProcessStripeWebhook implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    public $event;
    public $email;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($event, $email)
    {
        $this->event = $event;
        $this->email = $email;
    }

    private function createAndSavePdf($registration)
    {
        try {
            $identifier = uniqid();
            $path = storage_path('app/downloads/pdf/' . $identifier . '.pdf');   
            $calculation = (new PriceCalculator)->execute(
                $registration, 
                $registration->pricing_group_price, 
                $registration->attendees,
                $registration->extras
            );
            $pdf = Pdf::loadView('pdf.invoice', ['registration' => $registration, 'calculation' => $calculation]);
            $pdf->save(storage_path('/app/downloads/pdf/' . $identifier . '.pdf'));
            return $path;
        } catch (\Throwable $ex) {
            Log::error($ex);
        }
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {

        DB::beginTransaction();

        try {

            $registration = Registration::where('id', $this->event->data->object->metadata->registration)
                ->with('attendees','extras','transactions')
                ->first();

            if ($registration) {
                $registration->status = 2;
                $registration->save();

                $transaction = $registration->transactions()->create([
                    'reference' => $this->event->data->object->id,
                    'gateway' => 'stripe',
                    'amount' => $this->event->data->object->amount / 100,
                ]);
                
                $registration->fresh();

                $pdf = $this->createAndSavePdf($registration);

                if ($pdf) {
                    // Send client email
                    Mail::to($registration->email)
                        ->later(now()->addMinutes(3), new SendRegistrationPaidClientEmail($registration, $pdf));

                    Mail::to($this->email)
                        ->later(now()->addMinutes(3), new SendRegistrationPaidAdminEmail($registration, $pdf));
                }

                DB::commit();
            } else {
                DB::rollback();
                $this->fail();
            }

        } catch (Exception $ex) {
            Log::error($ex);
            DB::rollback();
            $this->fail();
        }
    }
}
