<?php

namespace App\Nova\Actions;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Support\Collection;
use Laravel\Nova\Actions\Action;
use Laravel\Nova\Fields\ActionFields;
use Illuminate\Support\Facades\Storage;
use Laravel\Nova\Fields\Text;
use Illuminate\Support\Facades\Mail;
use PhpOffice\PhpSpreadsheet\Reader\Csv;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;

use App\Models\Registration;

use App\Jobs\PurgeData;

use Log;
use Schema;

class RegistrationsDownloadAction extends Action
{
    use InteractsWithQueue, Queueable;
    
    private function getExtraFields($registrations)
    {
        $fields = [];

        foreach($registrations as $registration) {
            if ($registration->extra) {
                foreach(array_keys($registration->extra) as $key)
                {
                    if (!in_array($key, $fields)) {
                        $fields[] = $key;
                    }
                }
            }
        }

        return $fields;
    }

    private function sortRowToHeader($headers, $row)
    {
        $data = [];

        foreach($headers as $header) {
            $data[$header] = $row[$header] ?? NULL;
        }

        return $data;
    }

    private function getModelSchema()
    {    
        $remove = ['extra'];

        return array_filter(Schema::getColumnListing((new Registration)->getTable()), function($value, $key) use ($remove) {
            return !in_array($value, $remove);
        }, ARRAY_FILTER_USE_BOTH);
    }

    private function replaceRelationData($headers, $row, $registration)
    {
        $lookups = [
            'page_id' => ['object' => 'page','data' => 'name'],
            'country_id' => ['object' => 'country','data' => 'name'],
            'job_function_id' => ['object' => 'job_function','data' => 'name'],
            'company_type_id' => ['object' => 'company_type','data' => 'name'],
        ];

        $data = [];

        foreach($row as $key => $value) {
            if (in_array($key, array_keys($lookups))) {
                if (isset($registration->{$lookups[$key]['object']})) {
                    $data[$key] = $registration->{$lookups[$key]['object']}->{$lookups[$key]['data']};
                } else {
                    $data[$key] = $value;
                }
            }else {
                $data[$key] = $value;
            }
        }

        return $data;
    }

    private function createCSV($identifier, $registrations)
    {
        try {
            $path = storage_path('app/downloads/data/' . $identifier . '.csv');

            $headers = array_merge($this->getModelSchema(), $this->getExtraFields($registrations));

            sort($headers);

            $file = fopen($path, 'w');

            if ($file) {

                fputcsv($file, $headers, ",", '"');

                foreach($registrations as $registration)
                {
                    $row = $registration->toArray();

                    unset($row['extra']);

                    if ($registration->extra)
                    {
                        foreach(array_keys($registration->extra) as $key)
                        {
                            if (isset($registration->extra[$key])) {
                                $row[$key] = $registration->extra[$key];
                            } else {
                                $row[$key] = NULL;
                            }
                        }
                    }

                    $row = $this->sortRowToHeader($headers, $row);

                    $row = $this->replaceRelationData($headers, $row, $registration);
                       
                    fputcsv($file, $row, ",", '"');
                }
            }

            fclose($file);     
            
            return true;
        } catch (Exception $ex) {
            Log::error(ex);
        }
    }

    private function createXLSX($identifier)
    {
        try {
            $paths = [
                'csv' => storage_path('/app/downloads/data/'. $identifier .'.csv'),
                'xlsx' => storage_path('/app/downloads/data/'. $identifier .'.xlsx')
            ];
            $reader = new Csv();
            $encoding = Csv::guessEncoding($paths['csv']);
            $reader->setInputEncoding($encoding);
            $reader->setDelimiter(',');
            $reader->setEnclosure('"');
            $spreadsheet = $reader->load($paths['csv']);

            $writer = new Xlsx($spreadsheet);
            $writer->save($paths['xlsx']);
            
            return true;
        } catch (Exception $ex) {
            Log::error(ex);
        }
    }    


    /**
     * Perform the action on the given models.
     *
     * @param  \Laravel\Nova\Fields\ActionFields  $fields
     * @param  \Illuminate\Support\Collection  $models
     * @return mixed
     */
    public function handle(ActionFields $fields, Collection $models)
    {
        if (!$models) {
            return Action::danger('No registrations to download');
        }

        $identifier = uniqid();

        $ids = [];

        foreach($models as $model) {
            $ids[] = $model->id;
        }

        $registrations = Registration::whereIn('id', $ids)->with('page', 'country', 'job_function', 'company_type')->get();

        $created = $this->createCSV($identifier, $registrations);

        if (!$created) {
            return Action::danger('No CSV was created');
        }

        $converted = $this->createXLSX($identifier);

        if (!$converted) {
            return Action::danger('No XLSX was created');
        }

        PurgeData::dispatch()->delay(now()->addMinutes(10));

        return Action::download(config('app.url') . '/downloads/' . $identifier, $identifier . '.xlsx');
    }

    /**
     * Get the fields available on the action.
     *
     * @return array
     */
    public function fields()
    {
        return [];
    }
}
