<?php

namespace App\Nova;

use Illuminate\Http\Request;
use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\BelongsTo;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\Slug;
use Laravel\Nova\Fields\Textarea;
use Laravel\Nova\Fields\Markdown;
use Laravel\Nova\Fields\Select;
use Laravel\Nova\Fields\Image;
use Laravel\Nova\Fields\DateTime;
use Laravel\Nova\Fields\HasMany;
use Laravel\Nova\Panel;
use Laravel\Nova\Fields\Boolean;
use Techevents\PageLayout\PageLayout;

use Laravel\Nova\Http\Requests\NovaRequest;

use App\Nova\Actions\PageCopyAction;
use App\Nova\Actions\ChangeStatusActiveAction;
use App\Nova\Actions\ChangeStatusInActiveAction;

use App\Nova\Filters\ActivePageFilter;

class Page extends Resource
{
    /**
     * The model the resource corresponds to.
     *
     * @var string
     */
    public static $model = \App\Models\Page::class;

    /**
     * The group in which this resource belongs to.
     */
    public static $group = 'Events & Pages';

    /**
     * The single value that should be used to represent the resource when being displayed.
     *
     * @var string
     */
    public static $title = 'name';

    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [
        'id', 'name', 'slug', 'city'
    ];

    /**
     * Get the fields displayed by the resource.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function fields(Request $request)
    {
        return [
            new Panel('Page', $this->pageFields()),
            new Panel('Location', $this->locationFields()),
            new Panel('Date & Time', $this->dateFields()),
            new Panel('Portal', $this->portalFields()),     
            new Panel('Associated Content', $this->associatedContentFields()),
            new Panel('Layout', $this->layoutFields()),     

        ];
    }

    protected function pageFields()
    {
        return [
            ID::make(__('ID'), 'id'),            
            Text::make(__('Name'), 'name')
                ->rules('required', 'max:200'),       
            Slug::make(__('Slug'), 'slug')
                ->hideFromIndex()
                ->from('name')
                ->creationRules('required', 'max:200', 'unique:pages,slug')
                ->updateRules('required', 'max:200', 'unique:pages,slug,{{resourceId}}'),     
            Text::make(__('Meta Title'), 'title')
                ->hideFromIndex()
                ->rules('required', 'max:200'),  
            Text::make(__('Meta Description'), 'description')
                ->hideFromIndex()
                ->rules('required', 'max:400'),        
            Text::make('Preview', function () {
               return '<a href="/page/'. $this->slug .'?preview=true" title="Preview page" target="_blank" style="color:rgb(190, 31, 35);font-weight:bold;text-decoration:none">Preview</a>';
            })->asHtml(),              
            Select::make(__('Status'), 'status')
                ->options([
                    0 => 'Inactive', 
                    1 => 'Active'
                ])
                ->displayUsingLabels()
                ->rules('required'),                
        ];
    } 

    protected function locationFields()
    {
        return [
            Text::make(__('City'), 'city')
                ->rules('required', 'max:100'),
            Text::make(__('Country'), 'country')
                ->rules('required', 'max:100'),     
            Markdown::make(__('Venue'), 'venue')
                ->hideFromIndex()
                ->rules('nullable'),
            Markdown::make(__('Travel'), 'travel')
                ->hideFromIndex()
                ->rules('nullable'),
            Markdown::make(__('Accommodation'), 'accommodation')
                ->hideFromIndex()
                ->rules('nullable'),
            Markdown::make(__('Things To Do'), 'things_to_do')
                ->hideFromIndex()
                ->rules('nullable'),
            Markdown::make(__('Restrictions'), 'restrictions')
                ->hideFromIndex()
                ->rules('nullable'),
            Text::make(__('Longitude'), 'longitude')
                ->hideFromIndex()
                ->rules('required', 'numeric'),    
            Text::make(__('Latitude'), 'latitude')
                ->hideFromIndex()
                ->rules('required', 'numeric'),    
        ];
    }

    protected function dateFields()
    {
        return [
            DateTime::make(__('Starts At'), 'starts_at')
                ->rules('required'),
            DateTime::make(__('Ends At'), 'ends_at')
                ->rules('required'),           
        ];
    }

    protected function portalFields()
    {
        return [
            Boolean::make(__('Activate Portal?'), 'portal'),
            Markdown::make(__('Portal Text'), 'portal_text')
                ->hideFromIndex()
                ->rules('nullable'),
            Image::make(__('Floorplan'), 'floorplan')
                ->disk('s3')
                ->nullable()
                ->rules('nullable', 'mimes:jpg,svg'),
        ];
    }

    protected function associatedContentFields()
    {
        return [
            HasMany::make('Banners'),
            HasMany::make('Menu Items', 'menuitems', MenuItem::class),
            HasMany::make('Speakers'),
            HasMany::make('Judges'),
            HasMany::make('Sponsors'),
            HasMany::make('Expectations'),
            HasMany::make('Galleries'),
            HasMany::make('Programmes'),
            HasMany::make('Categories'),
            HasMany::make('Registrations'),
            HasMany::make('Hotels'),
            HasMany::make('Travels'),
            HasMany::make('Todos'),
            HasMany::make('Pricing Extras'),
        ];
    }

    protected function layoutFields()
    {
        return [
            PageLayout::make(__('Layout'), 'layout')
                ->withMeta(['settings' => ['blocks' => 'page']])
                ->stacked()
                ->hideFromIndex()
                ->hideFromDetail(),
        ];
    }

    /**
     * Get the cards available for the request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function cards(Request $request)
    {
        return [];
    }

    /**
     * Get the filters available for the resource.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function filters(Request $request)
    {
        return [
            new ActivePageFilter
        ];
    }

    /**
     * Get the lenses available for the resource.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function lenses(Request $request)
    {
        return [];
    }

    /**
     * Get the actions available for the resource.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function actions(Request $request)
    {
        return [
            new PageCopyAction,
            new ChangeStatusActiveAction,
            new ChangeStatusInActiveAction
        ];
    }
}
