<?php

namespace App\Nova;

use Illuminate\Http\Request;
use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\Boolean;
use Laravel\Nova\Fields\BelongsTo;
use Laravel\Nova\Fields\KeyValue;
use Laravel\Nova\Fields\HasMany;
use Laravel\Nova\Fields\Select;
use Laravel\Nova\Http\Requests\NovaRequest;

use App\Nova\Filters\RegistrationEventFilter;
use App\Nova\Filters\RegistrationPaidFilter;
use App\Nova\Filters\RegistrationNewsletterFilter;
use App\Nova\Filters\RegistrationReceptionFilter;
use App\Nova\Filters\RegistrationSponsorshipFilter;

use App\Nova\Actions\RegistrationsDownloadAction;
use App\Nova\Actions\CreatePortalUserAction;
use App\Nova\Actions\CreatePortalUserFromRegistrationAction;
use App\Nova\Actions\CreateAttendeeFromRegistrationAction;

class Registration extends Resource
{
    /**
     * The model the resource corresponds to.
     *
     * @var string
     */
    public static $model = \App\Models\Registration::class;

    /**
     * The group in which this resource belongs to.
     */
    public static $group = 'Registrations';

    /**
     * The number of resources to show per page via relationships.
     *
     * @var int
     */
    public static $perPageViaRelationship = 60;

    /**
     * Get the value that should be displayed to represent the resource.
     *
     * @return string
     */
    public function title()
    {
        return $this->forename . ' ' . $this->surname . ' (' . $this->page->name . ')';
    }

    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [
        'id','forename','surname','email','phone','company','promo'
    ];

    /**
     * Get the fields displayed by the resource.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function fields(Request $request)
    {
        return [
            ID::make(__('ID'), 'id')->sortable(),
            BelongsTo::make('Page'),
            BelongsTo::make('Pricing Group Price'),
            Text::make(__('Title'), 'title')
                ->rules('required', 'max:10'),
            Text::make(__('Forename'), 'forename')
                ->rules('required', 'max:50'),
            Text::make(__('Surname'), 'surname')
                ->rules('required', 'max:50'),    
            Text::make(__('Email'), 'email')
                ->rules('required', 'max:200', 'email'),    
            Text::make(__('Phone'), 'phone')
                ->hideFromIndex()
                ->rules('nullable', 'max:30'),
            Text::make(__('Company'), 'company')
                ->rules('nullable', 'max:100'),
            BelongsTo::make('Company Type')
                ->nullable()
                ->hideFromIndex(),
            BelongsTo::make('Job Function')
                ->nullable()
                ->hideFromIndex(),
            Text::make(__('Job Role'), 'job_role')
                ->hideFromIndex()
                ->rules('nullable', 'max:100'),     
            BelongsTo::make('Country')
                ->nullable()
                ->hideFromIndex(),
            Text::make(__('Promo'), 'promo')
                ->hideFromIndex()
                ->rules('nullable', 'max:100'),    
            KeyValue::make(__('Extra'), 'extra')
                ->rules('nullable'),                      
            Boolean::make(__('Newsletter'), 'newsletter')
                ->hideFromIndex()
                ->trueValue(1)
                ->falseValue(0),
            Boolean::make(__('Reception'), 'reception')
                ->hideFromIndex()
                ->trueValue(1)
                ->falseValue(0),
            Boolean::make(__('Sponsorship'), 'sponsorship')
                ->hideFromIndex()
                ->trueValue(1)
                ->falseValue(0), 
            Boolean::make(__('Portal'), 'portal')
                ->hideFromIndex()
                ->trueValue(1)
                ->falseValue(0),
            Select::make(__('Status'), 'status')
                ->options([
                    0 => 'Unpaid',
                    1 => 'Free Of Charge',
                    2 => 'Paid'
                ])
                ->displayUsingLabels()
                ->rules('required'),
            HasMany::make('Attendees'), 
            HasMany::make('Extras'),     
            HasMany::make('Transactions')     
        ];
    }

    /**
     * Get the cards available for the request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function cards(Request $request)
    {
        return [];
    }

    /**
     * Get the filters available for the resource.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function filters(Request $request)
    {
        return [
            new RegistrationEventFilter,
            new RegistrationPaidFilter,
            new RegistrationNewsletterFilter,
            new RegistrationReceptionFilter,
            new RegistrationSponsorshipFilter
        ];
    }

    /**
     * Get the lenses available for the resource.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function lenses(Request $request)
    {
        return [];
    }

    /**
     * Get the actions available for the resource.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function actions(Request $request)
    {
        return [
            new RegistrationsDownloadAction,
            new CreatePortalUserAction,
            new CreatePortalUserFromRegistrationAction,
            new CreateAttendeeFromRegistrationAction
        ];
    }
}
