<?php

namespace Tests\Browser;

use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Foundation\Testing\DatabaseTruncation;

use Laravel\Dusk\Browser;
use Tests\DuskTestCase;

use App\Models\Page;
use App\Models\Form;
use App\Models\PricingGroup;
use App\Models\PricingGroupPrice;

class PageFormTest extends DuskTestCase
{
    use DatabaseTruncation;

    public function test_minimum_fields_required_not_present()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([])
        ])->create();

        $page = Page::factory()->state([
            'layout' => json_encode([
                [
                    "id" => "form1",
                    "form" => $form->id,
                    "name" => "Registration",
                    "type" => "blocks.form-block",
                    "blocks" => "page"
                ],                
            ])
        ])->create();

        $this->browse(function (Browser $browser) use ($page, $form) {
            $browser->visit('/page/' . $page->slug)
                ->assertSee(strtoupper($form->heading))
                ->press('@submit')
                ->pause(1000)
                ->assertSee('The title field is required')
                ->assertSee('The forename field is required')
                ->assertSee('The surname field is required')
                ->assertSee('The email field is required');
        });        

    }

    public function test_minimum_fields_required_present(): void
    {
        $components = [
            [
                "id" => "form1",
                "form" => 1,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();
        

        $form = Form::factory()->state([
            'id' => 1,
            'type' => 'registration',
            'url' => '/page/' . $page->slug . '/checkout',
            'fields' => json_encode([
                [
                    "id" => "14211f6f347",
                    "name" => "Pricing Block",
                    "type" => "blocks.pricing-block",
                    "group" => 1
                ],                  
                [
                    "id" => "field1",
                    "name" => "Title",
                    "slug" => "title",
                    "type" => "inputs.text-input",
                    "label" => "Title",                    
                ],                
                [
                    "id" => "field2",
                    "name" => "Forename",
                    "slug" => "forename",
                    "type" => "inputs.text-input",
                    "label" => "Forename",                    
                ],                
                [
                    "id" => "field3",
                    "name" => "Surname",
                    "slug" => "surname",
                    "type" => "inputs.text-input",
                    "label" => "Surname",                    
                ],                
                [
                    "id" => "field4",
                    "name" => "Email",
                    "slug" => "email",
                    "type" => "inputs.text-input",
                    "label" => "Email",                    
                ] 
            ])
        ])->create();

        $group = PricingGroup::factory()->state([
            'page_id' => $page
        ])->create();

        $price = PricingGroupPrice::factory()->state([
            'pricing_group_id' => $group->id
        ])->create();

        $this->browse(function (Browser $browser) use ($page) {
            $browser
                ->visit(route('page', ['page' => $page->slug]))
                ->type('@title', 'Mr')
                ->pause(1000)
                ->type('@forename', 'John')
                ->pause(1000)
                ->type('@surname', 'Doe')
                ->pause(1000)
                ->type('@email', 'test@example.com')
                ->pause(1000)
                ->press('@submit')
                ->pause(1000)
                ->assertPathIs('/page/' . $page->slug . '/checkout');
        });

        $this->assertDatabaseHas('registrations', [
            'title' => 'Mr',
            'forename' => 'John',
            'surname' => 'Doe',
            'email' => 'test@example.com',
            'status' => 0
        ]);

        // Because couldn't get mail testing facade to work
        $this->assertDatabaseHas('jobs', [
            'queue' => 'default'
        ]);
    }        

    public function test_field_below_min_length_validation(): void
    {
        $components = [
            [
                "id" => "form1",
                "form" => 1,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];    

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();

        $form = Form::factory()->state([
            'id' => 1,
            'type' => 'registration',
            'url' => '/page/' . $page->slug . '/checkout',
            'fields' => json_encode([
                [
                    "id" => "field1",
                    "name" => "Field1",
                    "slug" => "field1",
                    "type" => "inputs.text-input",
                    "label" => "Field1",  
                    "validation" => [
                        "min" => "4",
                        "max" => "",
                        "required" => "",
                        "type" => "string"
                    ]                  
                ],                
            ])
        ])->create();

        $group = PricingGroup::factory()->state([
            'page_id' => $page
        ])->create();

        $price = PricingGroupPrice::factory()->state([
            'pricing_group_id' => $group->id
        ])->create();    
        
        $this->browse(function (Browser $browser) use ($page) {
            $browser
                ->visit(route('page', ['page' => $page->slug]))
                ->type('@field1', '123')
                ->pause(1000)
                ->press('@submit')
                ->pause(1000)
                ->assertSee('The field1 must be at least 4 characters');
        });
    }

    public function test_field_above_max_length_validation(): void
    {
        $components = [
            [
                "id" => "form1",
                "form" => 1,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];    

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();

        $form = Form::factory()->state([
            'id' => 1,
            'type' => 'registration',
            'url' => '/page/' . $page->slug . '/checkout',
            'fields' => json_encode([
                [
                    "id" => "field1",
                    "name" => "Field1",
                    "slug" => "field1",
                    "type" => "inputs.text-input",
                    "label" => "Field1",  
                    "validation" => [
                        "min" => "",
                        "max" => "25",
                        "required" => "",
                        "type" => "string"
                    ]                  
                ],                
            ])
        ])->create();

        $group = PricingGroup::factory()->state([
            'page_id' => $page
        ])->create();

        $price = PricingGroupPrice::factory()->state([
            'pricing_group_id' => $group->id
        ])->create();    
        
        $this->browse(function (Browser $browser) use ($page) {
            $browser
                ->visit(route('page', ['page' => $page->slug]))
                ->type('@field1', str_repeat('a', 26))
                ->pause(1000)
                ->press('@submit')
                ->pause(1000)
                ->assertSee('The field1 must not be greater than 25 characters');
        });

    }

}
