<?php

namespace Tests\Feature\Portal\Livewire;

use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Tests\TestCase;
use Livewire\Livewire;

use App\Http\Livewire\Portal\Chat\ChatBox;

use App\Models\Page;
use App\Models\User;
use App\Models\Portal\Chat;

class ChatBoxTest extends TestCase
{
    use RefreshDatabase;

    public function test_chat_box_renders()
    {
        $auth_user = User::factory()->create();
        $secondary_user = User::factory()->create();

        $this->actingAs($auth_user);

        $page = Page::factory()->create();

        Livewire::test(ChatBox::class, ['page' => $page])
            ->assertViewIs('livewire.portal.chat.chat-box');
    }

    public function test_chat_box_displays_messages_from_auth_user()
    {
       $page = Page::factory()->create();

        $auth = User::factory()->create();
        $user = User::factory()->create();

        $chat = Chat::factory()->create([
            'page_id' => $page->id,
            'from_user_id' => $auth->id,
            'to_user_id' => $user->id
        ]);

        $this->actingAs($auth);

        Livewire::withQueryParams(['selected' => $user->id])
            ->test(ChatBox::class, ['page' => $page])
            ->assertSet('page', $page)
            ->assertSet('selected', $user->id)
            ->assertCount('chats', 1)
            ->assertSee('Chatting with ' . $user->forename)
            ->assertSee($chat->message);
    }    

    public function test_chat_box_displays_conversation()
    {
       $page = Page::factory()->create();

        $auth = User::factory()->create();
        $user = User::factory()->create();

        $chat1 = Chat::factory()->create([
            'page_id' => $page->id,
            'from_user_id' => $auth->id,
            'to_user_id' => $user->id
        ]);

        $chat2 = Chat::factory()->create([
            'page_id' => $page->id,
            'from_user_id' => $user->id,
            'to_user_id' => $auth->id
        ]);        

        $this->actingAs($auth);

        Livewire::withQueryParams(['selected' => $user->id])
            ->test(ChatBox::class, ['page' => $page])
            ->assertSet('page', $page)
            ->assertSet('selected', $user->id)
            ->assertCount('chats', 2)
            ->assertSee('Chatting with ' . $user->forename)
            ->assertSee($chat1->message)
            ->assertSee($chat2->message);
    }    

    public function test_chat_box_displays_no_messages()
    {
       $page = Page::factory()->create();

        $auth = User::factory()->create();
        $user = User::factory()->create();

        $this->actingAs($auth);

        Livewire::withQueryParams(['selected' => $user->id])
            ->test(ChatBox::class, ['page' => $page])
            ->assertSet('page', $page)
            ->assertSet('selected', $user->id)
            ->assertCount('chats', 0)
            ->assertSee('Chatting with ' . $user->forename)
            ->assertSee('No messages yet');
    }

    public function test_chat_box_send_chat()
    {
        $page = Page::factory()->create();

        $auth = User::factory()->create();
        $user = User::factory()->create();

        $this->actingAs($auth);

        Livewire::withQueryParams(['selected' => $user->id])
            ->test(ChatBox::class, ['page' => $page])
            ->set('message', 'Hello')
            ->call('send')
            ->assertSet('message', '')
            ->assertCount('chats', 1)
            ->assertSee('Hello');

        $this->assertDatabaseHas('chats', [
            'page_id' => $page->id,
            'from_user_id' => $auth->id,
            'to_user_id' => $user->id,
            'message' => 'Hello'
        ]);
    }

}