<?php

namespace Tests\Feature\Site\Livewire\Ecommerce;

use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Tests\TestCase;
use Livewire\Livewire;

use App\Models\Page;
use App\Models\PricingGroup;
use App\Models\PricingGroupPrice;
use App\Models\Registration;
use App\Models\PricingExtra;
use App\Models\Extra;

use App\Http\Livewire\Ecommerce\OrderButtons;

class OrderButtonsTests extends TestCase
{
    use RefreshDatabase;

    public function test_displays_ordering_buttons()
    {
        $page = Page::factory()->create();

        $group = PricingGroup::factory()->state([
            'page_id' => $page->id  
        ])->create();

        $price = PricingGroupPrice::factory()->create([
            'pricing_group_id' => $group->id
        ]);

        $attendees = [
            [
                'forename' => 'Test',
                'surname' => 'User',
                'email' => 'test@example.com'
            ]
        ];

        $registration = Registration::factory()->create();

        $extras = [];

        session([
            'registration' => $registration,
            'attendees' => $attendees,
            'extras' => []
        ]);

        Livewire::test(OrderButtons::class, ['page' => $page])
            ->assertSet('page', $page)
            ->assertSet('registration', $registration)
            ->assertSee('Cancel Registration')
            ->assertSeeHtml('Review & Pay');

    }

    public function test_redirects_to_page_on_cancel()
    {
        $page = Page::factory()->create();

        $group = PricingGroup::factory()->state([
            'page_id' => $page->id  
        ])->create();

        $price = PricingGroupPrice::factory()->create([
            'pricing_group_id' => $group->id
        ]);

        $attendees = [
            [
                'forename' => 'Test',
                'surname' => 'User',
                'email' => 'test@example.com'
            ]
        ];

        $registration = Registration::factory()->create();

        $extras = [];

        session([
            'registration' => $registration,
            'attendees' => $attendees,
            'extras' => []
        ]);

        Livewire::test(OrderButtons::class, ['page' => $page])
            ->assertSet('page', $page)
            ->call('cancel')
            ->assertRedirect(route('page', ['page' => $page->slug]));

    }

    public function test_saves_registration_and_single_attendee_redirects_to_payment()
    {
        $page = Page::factory()->create();

        $group = PricingGroup::factory()->state([
            'page_id' => $page->id  
        ])->create();

        $price = PricingGroupPrice::factory()->create([
            'pricing_group_id' => $group->id
        ]);

        $attendees = [
            [
                'forename' => 'Test',
                'surname' => 'User',
                'email' => 'test@example.com'
            ]
        ];

        $registration = Registration::factory()->create();

        $extras = [];

        session([
            'registration' => $registration,
            'attendees' => $attendees,
            'extras' => []
        ]);

        Livewire::test(OrderButtons::class, ['page' => $page])
            ->assertSet('page', $page)
            ->call('save')
            ->assertRedirect(route('page.checkout.payment', ['page' => $page->slug]));

        $this->assertDatabaseCount('registrations', 1);
        $this->assertDatabaseCount('attendees', 1);

        $this->assertDatabaseHas('attendees', [
            'forename' => 'Test',
            'surname' => 'User',
            'email' => 'test@example.com'
        ]);

        $this->assertDatabaseHas('registrations', [
            'id' => $registration->id,
            'status' => 0
        ]);

    }

    public function test_saves_registration_and_multiple_attendees_redirects_to_payment()
    {
        $page = Page::factory()->create();

        $group = PricingGroup::factory()->state([
            'page_id' => $page->id  
        ])->create();

        $price = PricingGroupPrice::factory()->create([
            'pricing_group_id' => $group->id
        ]);

        $attendees = [
            [
                'forename' => 'Test1',
                'surname' => 'User1',
                'email' => 'test1@example.com'
            ],
            [
                'forename' => 'Test2',
                'surname' => 'User2',
                'email' => 'test2@example.com'
            ]
        ];

        $registration = Registration::factory()->create();

        $extras = [];

        session([
            'registration' => $registration,
            'attendees' => $attendees,
            'extras' => []
        ]);

        Livewire::test(OrderButtons::class, ['page' => $page])
            ->assertSet('page', $page)
            ->call('save')
            ->assertRedirect(route('page.checkout.payment', ['page' => $page->slug]));

        $this->assertDatabaseCount('registrations', 1);
        $this->assertDatabaseCount('attendees', 2);

        $this->assertDatabaseHas('attendees', [
            'forename' => 'Test1',
            'surname' => 'User1',
            'email' => 'test1@example.com'
        ]);

        $this->assertDatabaseHas('attendees', [
            'forename' => 'Test2',
            'surname' => 'User2',
            'email' => 'test2@example.com'
        ]);

        $this->assertDatabaseHas('registrations', [
            'id' => $registration->id,
            'status' => 0
        ]);

    }

    public function test_saves_registration_and_single_attendee_with_extra_redirects_to_payment()
    {
        $page = Page::factory()->create();

        $registration = Registration::factory()->create();

        $group = PricingGroup::factory()->state([
            'page_id' => $page->id  
        ])->create();

        $price = PricingGroupPrice::factory()->create([
            'pricing_group_id' => $group->id
        ]);

        $pricing_extra = PricingExtra::factory()->state([
            'page_id' => $page->id
        ])->create();

        $extra = Extra::factory()->state([
            'registration_id' => $registration->id,
            'pricing_extra_id' => $pricing_extra->id,
            'quantity' => 1,
            'price' => $pricing_extra->price
        ])->create();


        $attendees = [
            [
                'forename' => 'Test',
                'surname' => 'User',
                'email' => 'test@example.com'
            ]
        ];

        $extras = [
            $extra->pricing_extra_id => $extra->quantity
        ];

        session([
            'registration' => $registration,
            'attendees' => $attendees,
            'extras' => $extras
        ]);

        Livewire::test(OrderButtons::class, ['page' => $page])
            ->assertSet('page', $page)
            ->call('save')
            ->assertRedirect(route('page.checkout.payment', ['page' => $page->slug]));

        $this->assertDatabaseCount('registrations', 1);
        $this->assertDatabaseCount('attendees', 1);
        $this->assertDatabaseCount('extras', 1);

        $this->assertDatabaseHas('attendees', [
            'forename' => 'Test',
            'surname' => 'User',
            'email' => 'test@example.com'
        ]);


        $this->assertDatabaseHas('extras', [
            'registration_id' => $registration->id,
            'pricing_extra_id' => $pricing_extra->id,
            'quantity' => 1,
            'price' => $pricing_extra->price
        ]);

        $this->assertDatabaseHas('registrations', [
            'id' => $registration->id,
            'status' => 0
        ]);

    }


}