<?php

namespace Tests\Feature\Site\Livewire;

use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Tests\TestCase;
use Livewire\Livewire;

use App\Models\Form;
use App\Models\Page;

use App\Http\Livewire\Forms\DynamicForm;
use App\Http\Livewire\Forms\Inputs\TextAreaInput;
use App\Http\Livewire\Forms\Inputs\TextInput;
use App\Http\Livewire\Forms\Inputs\CheckBoxInput;
use App\Http\Livewire\Forms\Inputs\PortalCheckBoxInput;
use App\Http\Livewire\Forms\Selects\CompanyTypeSelect;
use App\Http\Livewire\Forms\Selects\CountrySelect;
use App\Http\Livewire\Forms\Selects\JobFunctionSelect;
use App\Http\Livewire\Forms\Selects\RegionSelect;
use App\Http\Livewire\Forms\Selects\TitleSelect;

class FormFieldsLayoutRenderedTest extends TestCase
{
    use RefreshDatabase;

    public function test_form_renders_text_block()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "a9ad4f1bf5",
                    "name" => "Intro Text",
                    "type" => "blocks.text-block",
                    "content" => "Testing intro text"   
                ]
            ])
        ])->create();

        $components = [
            [
                "id" => "2056ca72c5",
                "form" => $form->id,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();
        
        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $components[0]])
            ->assertSet('page', $page)
            ->assertSet('component', $components[0])
            ->assertSee($form->heading)
            ->assertSee('Testing intro text');
    }


    public function test_form_renders_text_input()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "632d71f2ed",
                    "name" => "Field",
                    "slug" => "field",
                    "type" => "inputs.text-input",
                    "label" => "Field",                  
                ]
            ])
        ])->create();

        $components = [
            [
                "id" => "2056ca72c5",
                "form" => $form->id,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();

        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $components[0]])
            ->assertSet('page', $page)
            ->assertSet('component', $components[0])
            ->assertSeeLivewire(TextInput::class);       

    }

    public function test_form_renders_text_area()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "632d71f2ed",
                    "name" => "Field",
                    "slug" => "field",
                    "type" => "inputs.text-area-input",
                    "label" => "Field",                  
                ]
            ])
        ])->create();

        $components = [
            [
                "id" => "2056ca72c5",
                "form" => $form->id,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();

        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $components[0]])
            ->assertSet('page', $page)
            ->assertSet('component', $components[0])
            ->assertSeeLivewire(TextAreaInput::class);       

    }

    public function test_form_renders_check_box()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "632d71f2ed",
                    "name" => "Field",
                    "slug" => "field",
                    "type" => "inputs.check-box-input",
                    "label" => "Field",                  
                ]
            ])
        ])->create();

        $components = [
            [
                "id" => "2056ca72c5",
                "form" => $form->id,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();

        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $components[0]])
            ->assertSet('page', $page)
            ->assertSet('component', $components[0])
            ->assertSeeLivewire(CheckBoxInput::class);       

    }

    public function test_form_renders_portal_check_box()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "632d71f2ed",
                    "name" => "Field",
                    "slug" => "field",
                    "type" => "inputs.portal-check-box-input",
                    "label" => "Field",                  
                ]
            ])
        ])->create();

        $components = [
            [
                "id" => "2056ca72c5",
                "form" => $form->id,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();

        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $components[0]])
            ->assertSet('page', $page)
            ->assertSet('component', $components[0])
            ->assertSeeLivewire(PortalCheckBoxInput::class);       

    }

    public function test_form_renders_company_type_select()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "632d71f2ed",
                    "name" => "Field",
                    "slug" => "field",
                    "type" => "selects.company-type-select",
                    "label" => "Field",                  
                ]
            ])
        ])->create();

        $layout = [
            [
                "id" => "2056ca72c5",
                "form" => $form->id,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];

        $components = [
            [
                "id" => "2056ca72c5",
                "form" => $form->id,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();

        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $components[0]])
            ->assertSet('page', $page)
            ->assertSet('component', $components[0])
            ->assertSeeLivewire(CompanyTypeSelect::class);                
    }

    public function test_form_renders_country_select()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "632d71f2ed",
                    "name" => "Field",
                    "slug" => "field",
                    "type" => "selects.country-select",
                    "label" => "Field",                  
                ]
            ])
        ])->create();

        $components = [
            [
                "id" => "2056ca72c5",
                "form" => $form->id,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();

        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $components[0]])
            ->assertSet('page', $page)
            ->assertSet('component', $components[0])
            ->assertSeeLivewire(CountrySelect::class);                
    }

    public function test_form_renders_job_function_select()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "632d71f2ed",
                    "name" => "Field",
                    "slug" => "field",
                    "type" => "selects.job-function-select",
                    "label" => "Field",                  
                ]
            ])
        ])->create();

        $components = [
            [
                "id" => "2056ca72c5",
                "form" => $form->id,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();

        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $components[0]])
            ->assertSet('page', $page)
            ->assertSet('component', $components[0])
            ->assertSeeLivewire(JobFunctionSelect::class);                
    }

    public function test_form_renders_region_select()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "632d71f2ed",
                    "name" => "Field",
                    "slug" => "field",
                    "type" => "selects.region-select",
                    "label" => "Field",                  
                ]
            ])
        ])->create();

        $components = [
            [
                "id" => "2056ca72c5",
                "form" => $form->id,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();

        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $components[0]])
            ->assertSet('page', $page)
            ->assertSet('component', $components[0])
            ->assertSeeLivewire(RegionSelect::class);                
    }    

    public function test_form_renders_title_select()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "632d71f2ed",
                    "name" => "Field",
                    "slug" => "field",
                    "type" => "selects.title-select",
                    "label" => "Field",      
                    "options" => [
                        [
                            "label" => "Mr", 
                            "value" => "Mr"
                        ]
                    ]            
                ]
            ])
        ])->create();

        $components = [
            [
                "id" => "2056ca72c5",
                "form" => $form->id,
                "name" => "Test Form",
                "type" => "blocks.form-block",
                "blocks" => "page",
            ]
        ];

        $page = Page::factory()->state([
            'layout' => json_encode($components)
        ])->create();

        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $components[0]])
            ->assertSet('page', $page)
            ->assertSet('component', $components[0])
            ->assertSeeLivewire(TitleSelect::class);                
    }    

}