<?php

namespace Tests\Feature\Site\Livewire;

use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Tests\TestCase;
use Livewire\Livewire;

use App\Models\Page;
use App\Models\Homepage;
use App\Models\Form;

use App\Http\Livewire\Layout\PageLayout;
use App\Http\Livewire\Forms\DynamicForm;
use App\Http\Livewire\Forms\Inputs\TextInput;

class FormFieldsValidationTest extends TestCase
{
    use RefreshDatabase;

    public function test_form_text_input_validation_required()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "632d71f2ed",
                    "name" => "Field",
                    "slug" => "field",
                    "type" => "inputs.text-input",
                    "label" => "Field",
                    "validation" => [
                        "max" => "",
                        "min" => "",
                        "type" => "string",
                        "required" => true
                    ]                    
                ]
            ])
        ])->create();

        $page = Page::factory()->create();
        
        $component =  [
            "id" => "2056ca72c5",
            "form" => $form->id,
            "name" => "Registration",
            "type" => "blocks.form-block",
            "blocks" => "page",
        ];

        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $component])
            ->assertSet('page', $page)
            ->assertSet('component', $component)
            ->call('submit')
            ->assertHasErrors(['data.field' => 'required']);
    }

    public function test_form_text_input_validation_max_exceeded()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "632d71f2ed",
                    "name" => "Field",
                    "slug" => "field",
                    "type" => "inputs.text-input",
                    "label" => "Field",
                    "validation" => [
                        "max" => "100",
                        "min" => "",
                        "type" => "string",
                        "required" => true
                    ]                    
                ]
            ])
        ])->create();

        $page = Page::factory()->create();
        
        $component =  [
            "id" => "2056ca72c5",
            "form" => $form->id,
            "name" => "Registration",
            "type" => "blocks.form-block",
            "blocks" => "page",
        ];

        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $component])
            ->assertSet('page', $page)
            ->assertSet('component', $component)
            ->set('data', ['field' => 'a' . str_repeat('a', 101)])
            ->call('submit')
            ->assertHasErrors(['data.field' => 'max']);
    }

    public function test_form_text_input_validation_min_lessthan()
    {
        $form = Form::factory()->state([
            'fields' => json_encode([
                [
                    "id" => "632d71f2ed",
                    "name" => "Field",
                    "slug" => "field",
                    "type" => "inputs.text-input",
                    "label" => "Field",
                    "validation" => [
                        "max" => "",
                        "min" => "5",
                        "type" => "string",
                        "required" => true
                    ]                    
                ]
            ])
        ])->create();

        $page = Page::factory()->create();
        
        $component =  [
            "id" => "2056ca72c5",
            "form" => $form->id,
            "name" => "Registration",
            "type" => "blocks.form-block",
            "blocks" => "page",
        ];

        Livewire::test(DynamicForm::class, ['page' => $page, 'component' => $component])
            ->assertSet('page', $page)
            ->assertSet('component', $component)
            ->set('data', ['field' => 'a' . str_repeat('a', 4)])
            ->call('submit')
            ->assertHasNoErrors('data.field');
    }


// [{"id":"a9ad4f1bf5","name":"Intro Text","type":"blocks.text-block","content":"Register now and take advantage of our special 3-for-2 group bookings offer! To benefit from this offer, simply enter your details below to register to attend The Tech Capital's International Finance Forum 2024 on May 23, 2024, in London. You will be able to add extra attendees on the next page.\n\nRegistering for The Tech Capital's International Finance Forum will give you access to:\n\n* All live talks across two theatres and a chance to actively engage in the discussions;\n* Sessions with market presentations and global outlooks;\n* Access to in-person networking alongside global peers;\n* Light refreshments and drinks throughout the day;\n* A fork buffet lunch with industry colleagues;\n* Attendance of networking drinks reception after the forum on May 23.\n\nTo register for The Tech Capital Global Awards 2024 evening dinner, please register for the Forum below and add a pass for the Awards during registration in the next stage of registering for IFF24. To book a table, please reach out to iffawards@thetechcapital.co.uk and a member of the team will be in touch shortly.\n\nRegistering for The Tech Capital's Global Awards will give you access to:\n\n* Pre-dinner networking drinks reception;\n* Three-course sit-down meal in one of London's greatest halls with beverages throughout;\n* Access to private networking with 150 industry peers;\n* Celebrate and recognise outstanding achievements in the field of digital infrastructure finance\n\nNote that there is a charge to attend this event which will be collected at the next stage. "},{"id":"14211f6f347","name":"Pricing Block","type":"blocks.pricing-block","group":1},{"id":"8ea92b11b4","name":"Ticket Caveat","type":"blocks.text-block","content":"Tickets are non-refundable but may be passed on to another colleague in the same company at no extra cost until December 31, 2023."},{"id":"8dc17c109d","name":"Title","slug":"title","type":"selects.title-select","label":"Title","options":[{"label":"Mr","value":"Mr"},{"label":"Mrs","value":"Mrs"},{"label":"Ms","value":"Ms"},{"label":"Dr","value":"Dr"},{"label":"Prof","value":"Prof"},{"label":"Mx","value":"Mx"}],"validation":{"max":10,"min":2,"type":"string","required":true}},{"id":"632d71f2ed","name":"Forename","slug":"forename","type":"inputs.text-input","label":"Forename","validation":{"max":"","min":"","type":"string","required":true}},{"id":"128bce414f0","name":"Surname","slug":"surname","type":"inputs.text-input","label":"Surname","validation":{"max":"","min":"","type":"string","required":true}},{"id":"d6b6c30a3","name":"Email","slug":"email","type":"inputs.text-input","label":"Email","validation":{"max":"","min":"","type":"email","required":true}},{"id":"3768a24977","name":"PA Email","slug":"pa_email","type":"inputs.text-input","label":"PA \/ Assistants Email","validation":{"type":"string","required":false}},{"id":"122647a7941","name":"Phone","slug":"phone","type":"inputs.text-input","label":"Phone","validation":{"max":"","min":"","type":"string","required":true}},{"id":"807c84fa34","name":"Company","slug":"company","type":"inputs.text-input","label":"Company","validation":{"max":"","min":"","type":"string","required":true}},{"id":"1465ce37d49","name":"Company Type ID","slug":"company_type_id","type":"selects.company-type-select","label":"Company Type","model":"App\\Models\\CompanyType","validation":{"required":true}},{"id":"16fb67b47c8","name":"Job Function ID","slug":"job_function_id","type":"selects.job-function-select","label":"Job Function","model":"App\\Models\\JobFunction","validation":{"required":true}},{"id":"f49ede6258","name":"Job Role","slug":"job_role","type":"inputs.text-input","label":"Role \/ Job Title","validation":{"max":"","min":"","type":"string","required":true}},{"id":"81635bde6d","name":"Country ID","slug":"country_id","type":"selects.country-select","label":"Country","model":"App\\Models\\Country","validation":{"required":true}},{"id":"f79281047d","name":"Promo","slug":"promo","type":"inputs.text-input","label":"Promotional Code","validation":{"type":"string","required":false}},{"id":"67d51ef364","name":"Newsletter","slug":"newsletter","type":"inputs.check-box-input","label":"I'd like to receive The Tech Capital's daily capital markets newsletter","validation":[]},{"id":"1001619fda7","name":"Sponsorship","slug":"sponsorship","type":"inputs.check-box-input","label":"I would like to hear more about sponsorship opportunities","validation":[]}]
// [
//     {
//         "id": "a9ad4f1bf5",
//         "name": "Intro Text",
//         "type": "blocks.text-block",
//         "content": "Register now and take advantage of our special 3-for-2 group bookings offer! To benefit from this offer, simply enter your details below to register to attend The Tech Capital's International Finance Forum 2024 on May 23, 2024, in London. You will be able to add extra attendees on the next page.\n\nRegistering for The Tech Capital's International Finance Forum will give you access to:\n\n* All live talks across two theatres and a chance to actively engage in the discussions;\n* Sessions with market presentations and global outlooks;\n* Access to in-person networking alongside global peers;\n* Light refreshments and drinks throughout the day;\n* A fork buffet lunch with industry colleagues;\n* Attendance of networking drinks reception after the forum on May 23.\n\nTo register for The Tech Capital Global Awards 2024 evening dinner, please register for the Forum below and add a pass for the Awards during registration in the next stage of registering for IFF24. To book a table, please reach out to iffawards@thetechcapital.co.uk and a member of the team will be in touch shortly.\n\nRegistering for The Tech Capital's Global Awards will give you access to:\n\n* Pre-dinner networking drinks reception;\n* Three-course sit-down meal in one of London's greatest halls with beverages throughout;\n* Access to private networking with 150 industry peers;\n* Celebrate and recognise outstanding achievements in the field of digital infrastructure finance\n\nNote that there is a charge to attend this event which will be collected at the next stage. "
//     },
//     {
//         "id": "14211f6f347",
//         "name": "Pricing Block",
//         "type": "blocks.pricing-block",
//         "group": 1
//     },
//     {
//         "id": "8ea92b11b4",
//         "name": "Ticket Caveat",
//         "type": "blocks.text-block",
//         "content": "Tickets are non-refundable but may be passed on to another colleague in the same company at no extra cost until December 31, 2023."
//     },
//     {
//         "id": "8dc17c109d",
//         "name": "Title",
//         "slug": "title",
//         "type": "selects.title-select",
//         "label": "Title",
//         "options": [
//             {
//                 "label": "Mr",
//                 "value": "Mr"
//             },
//             {
//                 "label": "Mrs",
//                 "value": "Mrs"
//             },
//             {
//                 "label": "Ms",
//                 "value": "Ms"
//             },
//             {
//                 "label": "Dr",
//                 "value": "Dr"
//             },
//             {
//                 "label": "Prof",
//                 "value": "Prof"
//             },
//             {
//                 "label": "Mx",
//                 "value": "Mx"
//             }
//         ],
//         "validation": {
//             "max": 10,
//             "min": 2,
//             "type": "string",
//             "required": true
//         }
//     },
//     {
//         "id": "632d71f2ed",
//         "name": "Forename",
//         "slug": "forename",
//         "type": "inputs.text-input",
//         "label": "Forename",
//         "validation": {
//             "max": "",
//             "min": "",
//             "type": "string",
//             "required": true
//         }
//     },
//     {
//         "id": "128bce414f0",
//         "name": "Surname",
//         "slug": "surname",
//         "type": "inputs.text-input",
//         "label": "Surname",
//         "validation": {
//             "max": "",
//             "min": "",
//             "type": "string",
//             "required": true
//         }
//     },
//     {
//         "id": "d6b6c30a3",
//         "name": "Email",
//         "slug": "email",
//         "type": "inputs.text-input",
//         "label": "Email",
//         "validation": {
//             "max": "",
//             "min": "",
//             "type": "email",
//             "required": true
//         }
//     },
//     {
//         "id": "3768a24977",
//         "name": "PA Email",
//         "slug": "pa_email",
//         "type": "inputs.text-input",
//         "label": "PA \/ Assistants Email",
//         "validation": {
//             "type": "string",
//             "required": false
//         }
//     },
//     {
//         "id": "122647a7941",
//         "name": "Phone",
//         "slug": "phone",
//         "type": "inputs.text-input",
//         "label": "Phone",
//         "validation": {
//             "max": "",
//             "min": "",
//             "type": "string",
//             "required": true
//         }
//     },
//     {
//         "id": "807c84fa34",
//         "name": "Company",
//         "slug": "company",
//         "type": "inputs.text-input",
//         "label": "Company",
//         "validation": {
//             "max": "",
//             "min": "",
//             "type": "string",
//             "required": true
//         }
//     },
//     {
//         "id": "1465ce37d49",
//         "name": "Company Type ID",
//         "slug": "company_type_id",
//         "type": "selects.company-type-select",
//         "label": "Company Type",
//         "model": "App\\Models\\CompanyType",
//         "validation": {
//             "required": true
//         }
//     },
//     {
//         "id": "16fb67b47c8",
//         "name": "Job Function ID",
//         "slug": "job_function_id",
//         "type": "selects.job-function-select",
//         "label": "Job Function",
//         "model": "App\\Models\\JobFunction",
//         "validation": {
//             "required": true
//         }
//     },
//     {
//         "id": "f49ede6258",
//         "name": "Job Role",
//         "slug": "job_role",
//         "type": "inputs.text-input",
//         "label": "Role \/ Job Title",
//         "validation": {
//             "max": "",
//             "min": "",
//             "type": "string",
//             "required": true
//         }
//     },
//     {
//         "id": "81635bde6d",
//         "name": "Country ID",
//         "slug": "country_id",
//         "type": "selects.country-select",
//         "label": "Country",
//         "model": "App\\Models\\Country",
//         "validation": {
//             "required": true
//         }
//     },
//     {
//         "id": "f79281047d",
//         "name": "Promo",
//         "slug": "promo",
//         "type": "inputs.text-input",
//         "label": "Promotional Code",
//         "validation": {
//             "type": "string",
//             "required": false
//         }
//     },
//     {
//         "id": "67d51ef364",
//         "name": "Newsletter",
//         "slug": "newsletter",
//         "type": "inputs.check-box-input",
//         "label": "I'd like to receive The Tech Capital's daily capital markets newsletter",
//         "validation": []
//     },
//     {
//         "id": "1001619fda7",
//         "name": "Sponsorship",
//         "slug": "sponsorship",
//         "type": "inputs.check-box-input",
//         "label": "I would like to hear more about sponsorship opportunities",
//         "validation": []
//     }
// ]
    // From here can go on to test valaidation on other input types
}
