<?php

namespace Tests\Unit;

use PHPUnit\Framework\TestCase;

use App\Actions\Forms\GenerateValidationRules;

class GenerateValidationRulesTest extends TestCase
{

    public function test_check_no_fields_no_rules()
    {
        $fields = [];

        $result = (new GenerateValidationRules)->execute($fields);

        $this->assertEmpty($result);
    }

    public function test_check_fields_no_validation_no_rules()
    {
        $fields = [
            ['id' => '1234', 'name' => 'Test1', 'slug' => 'test1', 'type' => 'inputs.text-input'],
            ['id' => '5678', 'name' => 'Test2', 'slug' => 'test2', 'type' => 'inputs.text-input'],
        ];

        $result = (new GenerateValidationRules)->execute($fields);

        $this->assertEmpty($result);
    }

    public function test_check_fields_with_validation_rules()
    {
        $fields = [
            ['id' => '1234', 'name' => 'Test1', 'slug' => 'test1', 'type' => 'inputs.text-input', 'validation' => ['required' => 'true']],
            ['id' => '5678', 'name' => 'Test2', 'slug' => 'test2', 'type' => 'inputs.text-input', 'validation' => ['required' => 'true']],
        ];

        $result = (new GenerateValidationRules)->execute($fields);

        $this->assertEquals([
            'data.test1' => 'required',
            'data.test2' => 'required',
        ], $result);
    }

    public function test_check_field_with_validation_rules_empty_value_omitted()
    {
        $fields = [
            ['id' => '1234', 'name' => 'Test1', 'slug' => 'test1', 'type' => 'inputs.text-input', 'validation' => ['required' => '']],
            ['id' => '5678', 'name' => 'Test2', 'slug' => 'test2', 'type' => 'inputs.text-input', 'validation' => ['required' => 'true']],
        ];

        $result = (new GenerateValidationRules)->execute($fields);

        $this->assertEquals([
            'data.test2' => 'required',
        ], $result);
    }
    
    public function test_check_field_with_validation_rules_max_field()
    {
        $fields = [
            ['id' => '1234', 'name' => 'Test1', 'slug' => 'test1', 'type' => 'inputs.text-input', 'validation' => ['max' => 10]],
            ['id' => '5678', 'name' => 'Test2', 'slug' => 'test2', 'type' => 'inputs.text-input', 'validation' => ['max' => 20]],
        ];

        $result = (new GenerateValidationRules)->execute($fields);

        $this->assertEquals([
            'data.test1' => 'max:10',
            'data.test2' => 'max:20',
        ], $result);
    }

    public function test_check_field_with_validation_rules_min_field()
    {
        $fields = [
            ['id' => '1234', 'name' => 'Test1', 'slug' => 'test1', 'type' => 'inputs.text-input', 'validation' => ['min' => 10]],
            ['id' => '5678', 'name' => 'Test2', 'slug' => 'test2', 'type' => 'inputs.text-input', 'validation' => ['min' => 20]],
        ];

        $result = (new GenerateValidationRules)->execute($fields);

        $this->assertEquals([
            'data.test1' => 'min:10',
            'data.test2' => 'min:20',
        ], $result);
    }    

    public function test_check_field_with_validation_rules_accepted_field()
    {
        $fields = [
            ['id' => '1234', 'name' => 'Test1', 'slug' => 'test1', 'type' => 'inputs.text-input', 'validation' => ['accepted' => 'true']],
            ['id' => '5678', 'name' => 'Test2', 'slug' => 'test2', 'type' => 'inputs.text-input', 'validation' => ['accepted' => 'true']],
        ];

        $result = (new GenerateValidationRules)->execute($fields);

        $this->assertEquals([
            'data.test1' => 'accepted',
            'data.test2' => 'accepted',
        ], $result);
    }    
    
    public function test_check_field_with_validation_rules_default_field()
    {
        $fields = [
            ['id' => '1234', 'name' => 'Test1', 'slug' => 'test1', 'type' => 'inputs.text-input', 'validation' => ['default' => 'test1']],
            ['id' => '5678', 'name' => 'Test2', 'slug' => 'test2', 'type' => 'inputs.text-input', 'validation' => ['default' => 'test2']],
        ];

        $result = (new GenerateValidationRules)->execute($fields);

        $this->assertEquals([
            'data.test1' => 'test1',
            'data.test2' => 'test2',
        ], $result);
    }        

    public function test_check_field_with_validation_rules_multiple_rules()
    {
        $fields = [
            ['id' => '1234', 'name' => 'Test1', 'slug' => 'test1', 'type' => 'inputs.text-input', 'validation' => ['required' => 'true', 'max' => '10', 'min' => '5']],
            ['id' => '5678', 'name' => 'Test2', 'slug' => 'test2', 'type' => 'inputs.text-input', 'validation' => ['required' => 'true', 'max' => '20', 'min' => '10']],
        ];

        $result = (new GenerateValidationRules)->execute($fields);

        $this->assertEquals([
            'data.test1' => 'required|max:10|min:5',
            'data.test2' => 'required|max:20|min:10',
        ], $result);

    }

}