<?php

namespace Tests\Unit;

use Illuminate\Foundation\Testing\RefreshDatabase;

use Tests\TestCase;

use App\Models\Registration;
use App\Models\PricingGroupPrice;
use App\Models\Page;
use App\Models\PricingExtra;
use App\Models\Extra;
use App\Models\PromoCode;

use App\Actions\Ecommerce\PriceCalculator;

class PriceCalculatorTest extends TestCase
{
    use RefreshDatabase;

    public function test_no_promo_code_single_attendee()
    {
        $registration = Registration::factory()->state([
            'pricing_group_price_id' => null
        ])->create();

        $price = PricingGroupPrice::factory()->create();

        $attendees = [
            [
                'forename' => 'Test',
                'surname' => 'User',
                'email' => 'test@example.com'
            ]
        ];

        $extras = [];

        $result = (new PriceCalculator)->execute($registration, $price, $attendees, $extras);

        // Check result is array
        $this->assertIsArray($result);

        // Check result has the fields expected
        $this->assertArrayHasKey('price', $result);
        $this->assertArrayHasKey('attendees', $result);
        $this->assertArrayHasKey('extras', $result);
        $this->assertArrayHasKey('discount', $result);
        $this->assertArrayHasKey('amountExDiscount', $result);
        $this->assertArrayHasKey('amount', $result);
        $this->assertArrayHasKey('tax', $result);
        $this->assertArrayHasKey('amountPlusTax', $result);

        // Check fields have the expected values
        $this->assertEquals($price, $result['price']);

        $this->assertCount(count($attendees), $result['attendees']);
        $this->assertArrayHasKey('forename', $result['attendees'][0]);
        $this->assertArrayHasKey('surname', $result['attendees'][0]);
        $this->assertArrayHasKey('email', $result['attendees'][0]);
        $this->assertContains($attendees[0]['forename'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['surname'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['email'], $result['attendees'][0]);

        $this->assertCount(count($extras), $result['extras']);

        // No discount
        $this->assertEquals(0, $result['discount']);

        // Amount without discount is same as price as no discount
        $this->assertEquals($price->price * count($attendees), $result['amountExDiscount']);

        // amount is price * number of attendees
        $this->assertEquals($price->price * count($attendees), $result['amount']);

        // tax is 20% of amount (VAT)
        $this->assertEquals(($price->price * count($attendees)) * 0.2, $result['tax']);

        // amount plus tax is amount + 20% tax (VAT)
        $this->assertEquals(($price->price * count($attendees)) * 1.2, $result['amountPlusTax']);

    }

    public function test_no_promo_code_multiple_attendee()
    {
        $registration = Registration::factory()->state([
            'pricing_group_price_id' => null
        ])->create();

        $price = PricingGroupPrice::factory()->create();

        $attendees = [
            [
                'forename' => 'Test1',
                'surname' => 'User1',
                'email' => 'test1@example.com'
            ],
            [
                'forename' => 'Test2',
                'surname' => 'User2',
                'email' => 'test2@example.com'
            ]
        ];

        $extras = [];

        $result = (new PriceCalculator)->execute($registration, $price, $attendees, $extras);

        // Check result is array
        $this->assertIsArray($result);

        // Check result has the fields expected
        $this->assertArrayHasKey('price', $result);
        $this->assertArrayHasKey('attendees', $result);
        $this->assertArrayHasKey('extras', $result);
        $this->assertArrayHasKey('discount', $result);
        $this->assertArrayHasKey('amountExDiscount', $result);
        $this->assertArrayHasKey('amount', $result);
        $this->assertArrayHasKey('tax', $result);
        $this->assertArrayHasKey('amountPlusTax', $result);

        // Check fields have the expected values
        $this->assertEquals($price, $result['price']);

        $this->assertCount(count($attendees), $result['attendees']);
        $this->assertArrayHasKey('forename', $result['attendees'][0]);
        $this->assertArrayHasKey('surname', $result['attendees'][0]);
        $this->assertArrayHasKey('email', $result['attendees'][0]);
        $this->assertContains($attendees[0]['forename'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['surname'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['email'], $result['attendees'][0]);
        $this->assertArrayHasKey('forename', $result['attendees'][1]);
        $this->assertArrayHasKey('surname', $result['attendees'][1]);
        $this->assertArrayHasKey('email', $result['attendees'][1]);
        $this->assertContains($attendees[1]['forename'], $result['attendees'][1]);
        $this->assertContains($attendees[1]['surname'], $result['attendees'][1]);
        $this->assertContains($attendees[1]['email'], $result['attendees'][1]);

        $this->assertCount(count($extras), $result['extras']);

        // No discount
        $this->assertEquals(0, $result['discount']);

        // Amount without discount is same as price as no discount
        $this->assertEquals($price->price * count($attendees), $result['amountExDiscount']);

        // amount is price * number of attendees
        $this->assertEquals($price->price * count($attendees), $result['amount']);

        // tax is 20% of amount (VAT)
        $this->assertEquals(($price->price * count($attendees)) * 0.2, $result['tax']);

        // amount plus tax is amount + 20% tax (VAT)
        $this->assertEquals(($price->price * count($attendees)) * 1.2, $result['amountPlusTax']);

    }

    public function test_promo_code_discount_single_attendee()
    {
        $promo = PromoCode::factory()->state([
            'code' => 'TEST20',
            'discount' => 20
        ])->create();

        $registration = Registration::factory()->state([
            'pricing_group_price_id' => null,
            'promo' => 'TEST20'
        ])->create();

        $price = PricingGroupPrice::factory()->create();

        $attendees = [
            [
                'forename' => 'Test1',
                'surname' => 'User1',
                'email' => 'test1@example.com'
            ]
        ];

        $extras = [];

        $result = (new PriceCalculator)->execute($registration, $price, $attendees, $extras);

        // Check result is array
        $this->assertIsArray($result);

        // Check result has the fields expected
        $this->assertArrayHasKey('price', $result);
        $this->assertArrayHasKey('attendees', $result);
        $this->assertArrayHasKey('extras', $result);
        $this->assertArrayHasKey('discount', $result);
        $this->assertArrayHasKey('amountExDiscount', $result);
        $this->assertArrayHasKey('amount', $result);
        $this->assertArrayHasKey('tax', $result);
        $this->assertArrayHasKey('amountPlusTax', $result);

        // Check fields have the expected values
        $this->assertEquals($price, $result['price']);

        // Check attendees
        $this->assertCount(count($attendees), $result['attendees']);
        $this->assertArrayHasKey('forename', $result['attendees'][0]);
        $this->assertArrayHasKey('surname', $result['attendees'][0]);
        $this->assertArrayHasKey('email', $result['attendees'][0]);
        $this->assertContains($attendees[0]['forename'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['surname'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['email'], $result['attendees'][0]);

        $this->assertCount(count($extras), $result['extras']);

        // Discount at 20%
        $this->assertEquals(($price->price * ($promo->discount / 100)) * count($attendees), $result['discount']);

        // Amount without discount applied is same as price
        $this->assertEquals($price->price * count($attendees), $result['amountExDiscount']);

        // amount is price * number of attendees minus 20% discount
        $this->assertEquals(($price->price - ($price->price * ($promo->discount / 100))) * count($attendees), $result['amount']);

        // tax is 20% of amount (VAT)
        $this->assertEquals((($price->price - ($price->price * ($promo->discount / 100))) * count($attendees)) * 0.2, $result['tax']);

        // amount plus tax is amount + 20% tax (VAT)
        $this->assertEquals((($price->price - ($price->price * ($promo->discount / 100))) * count($attendees)) * 1.2, $result['amountPlusTax']);

    }

    public function test_promo_code_discount_multiple_attendee()
    {
        $promo = PromoCode::factory()->state([
            'code' => 'TEST20',
            'discount' => 20
        ])->create();

        $registration = Registration::factory()->state([
            'pricing_group_price_id' => null,
            'promo' => 'TEST20'
        ])->create();

        $price = PricingGroupPrice::factory()->create();

        $attendees = [
            [
                'forename' => 'Test1',
                'surname' => 'User1',
                'email' => 'test1@example.com'
            ],
            [
                'forename' => 'Test2',
                'surname' => 'User2',
                'email' => 'test2@example.com'
            ]            
        ];

        $extras = [];

        $result = (new PriceCalculator)->execute($registration, $price, $attendees, $extras);

        // Check result is array
        $this->assertIsArray($result);

        // Check result has the fields expected
        $this->assertArrayHasKey('price', $result);
        $this->assertArrayHasKey('attendees', $result);
        $this->assertArrayHasKey('extras', $result);
        $this->assertArrayHasKey('discount', $result);
        $this->assertArrayHasKey('amountExDiscount', $result);
        $this->assertArrayHasKey('amount', $result);
        $this->assertArrayHasKey('tax', $result);
        $this->assertArrayHasKey('amountPlusTax', $result);

        // Check fields have the expected values
        $this->assertEquals($price, $result['price']);

        // Check attendees
        $this->assertCount(count($attendees), $result['attendees']);
        $this->assertArrayHasKey('forename', $result['attendees'][0]);
        $this->assertArrayHasKey('surname', $result['attendees'][0]);
        $this->assertArrayHasKey('email', $result['attendees'][0]);
        $this->assertContains($attendees[0]['forename'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['surname'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['email'], $result['attendees'][0]);
        $this->assertArrayHasKey('forename', $result['attendees'][1]);
        $this->assertArrayHasKey('surname', $result['attendees'][1]);
        $this->assertArrayHasKey('email', $result['attendees'][1]);
        $this->assertContains($attendees[1]['forename'], $result['attendees'][1]);
        $this->assertContains($attendees[1]['surname'], $result['attendees'][1]);
        $this->assertContains($attendees[1]['email'], $result['attendees'][1]);

        $this->assertCount(count($extras), $result['extras']);

        // Discount at 20%
        $this->assertEquals(($price->price * ($promo->discount / 100)) * count($attendees), $result['discount']);

        // Amount without discount applied is same as price
        $this->assertEquals($price->price * count($attendees), $result['amountExDiscount']);

        // amount is price * number of attendees minus 20% discount
        $this->assertEquals(($price->price - ($price->price * ($promo->discount / 100))) * count($attendees), $result['amount']);

        // tax is 20% of amount (VAT)
        $this->assertEquals((($price->price - ($price->price * ($promo->discount / 100))) * count($attendees)) * 0.2, $result['tax']);

        // amount plus tax is amount + 20% tax (VAT)
        $this->assertEquals((($price->price - ($price->price * ($promo->discount / 100))) * count($attendees)) * 1.2, $result['amountPlusTax']);
    }    

    public function test_no_promo_code_extra_single_attendee()
    {
        $page = Page::factory()->create();

        $registration = Registration::factory()->state([
            'page_id' => $page,
            'pricing_group_price_id' => null
        ])->create();

        $price = PricingGroupPrice::factory()->create();

        $price_extra = PricingExtra::factory()->state([
            'page_id' => $page
        ])->create();

        $extra = Extra::factory()->state([
            'registration_id' => $registration->id,
            'pricing_extra_id' => $price_extra->id,
            'quantity' => 1
        ])->create();

        $attendees = [
            [
                'forename' => 'Test1',
                'surname' => 'User1',
                'email' => 'test1@example.com'
            ],
        ];

        $extras = collect([$extra]);

        $result = (new PriceCalculator)->execute($registration, $price, $attendees, $extras);

        // Check result is array
        $this->assertIsArray($result);

        // Check result has the fields expected
        $this->assertArrayHasKey('price', $result);
        $this->assertArrayHasKey('attendees', $result);
        $this->assertArrayHasKey('extras', $result);
        $this->assertArrayHasKey('discount', $result);
        $this->assertArrayHasKey('amountExDiscount', $result);
        $this->assertArrayHasKey('amount', $result);
        $this->assertArrayHasKey('tax', $result);
        $this->assertArrayHasKey('amountPlusTax', $result);

        // Check fields have the expected values
        $this->assertEquals($price, $result['price']);

        $this->assertEquals($extra, $result['extras'][0]);

        // Check attendees
        $this->assertArrayHasKey('forename', $result['attendees'][0]);
        $this->assertArrayHasKey('surname', $result['attendees'][0]);
        $this->assertArrayHasKey('email', $result['attendees'][0]);
        $this->assertContains($attendees[0]['forename'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['surname'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['email'], $result['attendees'][0]);

        $this->assertCount(count($extras), $result['extras']);

        // No discount
        $this->assertEquals(0, $result['discount']);

        // Amount without discount is same as amount
        $this->assertEquals(($price->price * count($attendees)) + ($price_extra->price * $extra->quantity) , $result['amountExDiscount']);        

        // Amount will be price * number of attendees + extra price * quantity
        $this->assertEquals(($price->price * count($attendees)) + ($price_extra->price * $extra->quantity) , $result['amount']);

        // tax is 20% of amount (VAT)
        $this->assertEquals((($price->price * count($attendees)) + ($price_extra->price * $extra->quantity)) * 0.2, $result['tax']);

        // amount plus tax is amount + 20% tax (VAT)
        $this->assertEquals((($price->price * count($attendees)) + ($price_extra->price * $extra->quantity)) * 1.2, $result['amountPlusTax']);

    }

    public function test_promo_code_extra_single_attendee()
    {
        $promo = PromoCode::factory()->state([
            'code' => 'TEST20',
            'discount' => 20
        ])->create();

        $page = Page::factory()->create();

        $registration = Registration::factory()->state([
            'page_id' => $page,
            'pricing_group_price_id' => null,
            'promo' => 'TEST20'
        ])->create();

        $price = PricingGroupPrice::factory()->create();

        $price_extra = PricingExtra::factory()->state([
            'page_id' => $page
        ])->create();

        $extra = Extra::factory()->state([
            'registration_id' => $registration->id,
            'pricing_extra_id' => $price_extra->id,
            'price' => $price_extra->price,
            'quantity' => 1
        ])->create();

        $attendees = [
            [
                'forename' => 'Test1',
                'surname' => 'User1',
                'email' => 'test1@example.com'
            ],
        ];

        $extras = collect([$extra]);

        $result = (new PriceCalculator)->execute($registration, $price, $attendees, $extras);

        // Check result is array
        $this->assertIsArray($result);

        // Check result has the fields expected
        $this->assertArrayHasKey('price', $result);
        $this->assertArrayHasKey('attendees', $result);
        $this->assertArrayHasKey('extras', $result);
        $this->assertArrayHasKey('discount', $result);
        $this->assertArrayHasKey('amountExDiscount', $result);
        $this->assertArrayHasKey('amount', $result);
        $this->assertArrayHasKey('tax', $result);
        $this->assertArrayHasKey('amountPlusTax', $result);

        // Check fields have the expected values
        $this->assertEquals($price, $result['price']);

        $this->assertEquals($extra, $result['extras'][0]);

        // Check attendees
        $this->assertArrayHasKey('forename', $result['attendees'][0]);
        $this->assertArrayHasKey('surname', $result['attendees'][0]);
        $this->assertArrayHasKey('email', $result['attendees'][0]);
        $this->assertContains($attendees[0]['forename'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['surname'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['email'], $result['attendees'][0]);

        $this->assertCount(count($extras), $result['extras']);

        // Discount of 20%
        $this->assertEquals((($price->price * count($attendees)) + ($price_extra->price * $extra->quantity)) * ($promo->discount / 100), $result['discount']);

        // Amount without discount is same as amount
        $this->assertEquals(($price->price * count($attendees)) + ($price_extra->price * $extra->quantity) , $result['amountExDiscount']);        

        // Amount will be price * number of attendees + extra price * quantity - discount
        $this->assertEquals((($price->price * count($attendees)) + ($price_extra->price * $extra->quantity)) - (($price->price * count($attendees)) + ($price_extra->price * $extra->quantity)) * ($promo->discount / 100) , $result['amount']);

        // tax is 20% of amount (VAT)
        $this->assertEquals(((($price->price * count($attendees)) + ($price_extra->price * $extra->quantity)) - (($price->price * count($attendees)) + ($price_extra->price * $extra->quantity)) * ($promo->discount / 100)) * 0.2, $result['tax']);

        // // amount plus tax is amount + 20% tax (VAT)
        $this->assertEquals(((($price->price * count($attendees)) + ($price_extra->price * $extra->quantity)) - (($price->price * count($attendees)) + ($price_extra->price * $extra->quantity)) * ($promo->discount / 100)) * 1.2, $result['amountPlusTax']);

    }

    public function test_no_promo_code_single_attendee_buy_two_get_one_free()
    {
        $registration = Registration::factory()->state([
            'pricing_group_price_id' => null
        ])->create();

        $price = PricingGroupPrice::factory()->state([
            'offer' => 'buy-two-get-one-free'
        ])->create();

        $attendees = [
            [
                'forename' => 'Test1',
                'surname' => 'User',
                'email' => 'test1@example.com'
            ],
            [
                'forename' => 'Test1',
                'surname' => 'User',
                'email' => 'test1@example.com'
            ],
            [
                'forename' => 'Test1',
                'surname' => 'User',
                'email' => 'test1@example.com'
            ]                        
        ];

        $extras = [];

        $result = (new PriceCalculator)->execute($registration, $price, $attendees, $extras);

        // Check result is array
        $this->assertIsArray($result);

        // Check result has the fields expected
        $this->assertArrayHasKey('price', $result);
        $this->assertArrayHasKey('attendees', $result);
        $this->assertArrayHasKey('extras', $result);
        $this->assertArrayHasKey('discount', $result);
        $this->assertArrayHasKey('amountExDiscount', $result);
        $this->assertArrayHasKey('amount', $result);
        $this->assertArrayHasKey('tax', $result);
        $this->assertArrayHasKey('amountPlusTax', $result);

        // Check fields have the expected values
        $this->assertEquals($price, $result['price']);

        $this->assertCount(count($attendees), $result['attendees']);
        $this->assertArrayHasKey('forename', $result['attendees'][0]);
        $this->assertArrayHasKey('surname', $result['attendees'][0]);
        $this->assertArrayHasKey('email', $result['attendees'][0]);
        $this->assertContains($attendees[0]['forename'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['surname'], $result['attendees'][0]);
        $this->assertContains($attendees[0]['email'], $result['attendees'][0]);
        $this->assertArrayHasKey('forename', $result['attendees'][1]);
        $this->assertArrayHasKey('surname', $result['attendees'][1]);
        $this->assertArrayHasKey('email', $result['attendees'][1]);
        $this->assertContains($attendees[1]['forename'], $result['attendees'][1]);
        $this->assertContains($attendees[1]['surname'], $result['attendees'][1]);
        $this->assertContains($attendees[1]['email'], $result['attendees'][1]);
        $this->assertCount(count($extras), $result['extras']);
        $this->assertArrayHasKey('forename', $result['attendees'][2]);
        $this->assertArrayHasKey('surname', $result['attendees'][2]);
        $this->assertArrayHasKey('email', $result['attendees'][2]);
        $this->assertContains($attendees[2]['forename'], $result['attendees'][2]);
        $this->assertContains($attendees[2]['surname'], $result['attendees'][2]);
        $this->assertContains($attendees[2]['email'], $result['attendees'][2]);
        $this->assertCount(count($extras), $result['extras']);        

        // No discount
        $this->assertEquals(0, $result['discount']);

        // Amount without discount is same as price as no discount
        $this->assertEquals($price->price * (count($attendees) - 1), $result['amountExDiscount']);

        // amount is price * number of attendees
        $this->assertEquals($price->price * (count($attendees) - 1), $result['amount']);

        // tax is 20% of amount (VAT)
        $this->assertEquals(($price->price * (count($attendees) - 1)) * 0.2, $result['tax']);

        // amount plus tax is amount + 20% tax (VAT)
        $this->assertEquals(($price->price * (count($attendees) - 1)) * 1.2, $result['amountPlusTax']);

    }

}
