<?php

namespace Tests\Unit;

use Illuminate\Foundation\Testing\RefreshDatabase;

use Tests\TestCase;

use App\Models\Page;
use App\Models\PricingGroup;
use App\Models\PricingGroupPrice;

use App\Actions\Forms\ResolveRegistrationData;


class ResolveRegistrationDataTest extends TestCase
{
    use RefreshDatabase;

    public function test_resolves_registration_data()
    {
        $page = Page::factory()->create();
        $group = PricingGroup::factory()->state(['page_id' => $page->id])->create();
        $price = PricingGroupPrice::factory()->state(['pricing_group_id' => $group->id])->create();

        $properties = [
            'title', 'forename', 'surname', 'email'
        ];

        $input = [
            'title' => 'Mr',
            'forename' => 'John',
            'surname' => 'Doe',
            'email' => 'doe@example.com',
        ];

        $result = (new ResolveRegistrationData)->execute($page, $price, $properties, $input);

        $this->assertIsArray($result);
        $this->assertCount(7, $result); // input length + page_id + pricing_group_price_id + created_at - no extra field
        $this->assertArrayHasKey('title', $result);
        $this->assertEquals('Mr', $result['title']);
        $this->assertArrayHasKey('forename', $result);
        $this->assertEquals('John', $result['forename']);
        $this->assertArrayHasKey('surname', $result);
        $this->assertEquals('Doe', $result['surname']);
        $this->assertArrayHasKey('email', $result);
        $this->assertEquals('doe@example.com', $result['email']);
        $this->assertArrayHasKey('page_id', $result);
        $this->assertEquals($page->id, $result['page_id']);
        $this->assertArrayHasKey('pricing_group_price_id', $result);
        $this->assertEquals($price->id, $result['pricing_group_price_id']);
        $this->assertArrayHasKey('created_at', $result);
        $this->assertTrue(get_class($result['created_at'])=== 'Illuminate\Support\Carbon');
    }

    public function test_resolves_registration_data_adds_additional_data_to_extra_field()
    {
        $page = Page::factory()->create();
        $group = PricingGroup::factory()->state(['page_id' => $page->id])->create();
        $price = PricingGroupPrice::factory()->state(['pricing_group_id' => $group->id])->create();

        $properties = [
            'title', 'forename', 'surname', 'email'
        ];

        $input = [
            'title' => 'Mr',
            'forename' => 'John',
            'surname' => 'Doe',
            'email' => 'doe@example.com',
            'test' => 'value'
        ];

        $result = (new ResolveRegistrationData)->execute($page, $price, $properties, $input);

        $this->assertIsArray($result);
        $this->assertCount(8, $result); // Input length + extra field + page_id + pricing_group_price_id + created_at
        $this->assertArrayHasKey('extra', $result);
        $this->assertIsArray($result['extra']);
        $this->assertArrayHasKey('test', $result['extra']);
        $this->assertContains('value', $result['extra']);
    }

    public function test_resolves_registration_data_no_input()
    {
        $page = Page::factory()->create();
        $group = PricingGroup::factory()->state(['page_id' => $page->id])->create();
        $price = PricingGroupPrice::factory()->state(['pricing_group_id' => $group->id])->create();

        $properties = [
            'title', 'forename', 'surname', 'email'
        ];

        $input = [];

        $result = (new ResolveRegistrationData)->execute($page, $price, $properties, $input);

        $this->assertIsArray($result);
        $this->assertCount(3, $result); // No input so only page_id + pricing_group_price_id + created_at
        $this->assertArrayHasKey('page_id', $result);
        $this->assertEquals($page->id, $result['page_id']);
        $this->assertArrayHasKey('pricing_group_price_id', $result);
        $this->assertEquals($price->id, $result['pricing_group_price_id']);
        $this->assertArrayHasKey('created_at', $result);

        $this->assertTrue(get_class($result['created_at'])=== 'Illuminate\Support\Carbon');      

    }

}