<?php

namespace Tests\Unit;

use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Mail\Mailable;
use Illuminate\Support\Facades\Mail;

use Tests\TestCase;

use App\Models\Form;

use App\Actions\Forms\SendFormEmail;


class SendFormEmailTest extends TestCase
{
    use RefreshDatabase;

    public function test_queues_email_form_to_single_receipient()
    {
        Mail::fake();

        $form = Form::factory()->create();

        $email = new Mailable;

        $result = (new SendFormEmail)->execute($form, $email);

        Mail::assertQueued(Mailable::class);
        Mail::assertQueued(function (Mailable $mail) use ($form) {
            return $mail->hasTo($form->to);
        });
    }

    public function test_queues_email_form_to_multiple_receipient()
    {
        Mail::fake();

        $form = Form::factory()->state([
            'to' => 'test1@example.com,test2@example.com'
        ])->create();

        $email = new Mailable;

        $result = (new SendFormEmail)->execute($form, $email);

        Mail::assertQueued(Mailable::class);
        Mail::assertQueued(function (Mailable $mail) use ($form) {
            return $mail->hasTo('test1@example.com') && $mail->hasTo('test2@example.com');
        });
    }    

    public function test_queues_email_receipient_override()
    {
        Mail::fake();

        $form = Form::factory()->create();

        $email = new Mailable;

        $result = (new SendFormEmail)->execute($form, $email, ['test@example.com']);

        Mail::assertQueued(Mailable::class);
        Mail::assertQueued(function (Mailable $mail) use ($form) {
            return $mail->hasTo('test@example.com') && !$mail->hasTo($form->to);
        });
    }

    public function test_queues_no_form_to_default_receipients()
    {
        Mail::fake();

        $form = Form::factory()->state([
            'to' => null
        ])->create();

        $email = new Mailable;

        $result = (new SendFormEmail)->execute($form, $email, [], ['default@example.com']);

        Mail::assertQueued(Mailable::class);
        Mail::assertQueued(function (Mailable $mail) {
            return $mail->hasTo('default@example.com');
        });

    }
    
    public function test_queues_form_adds_single_cc()
    {
        Mail::fake();

        $form = Form::factory()->state([
            'cc' => 'test@example.com'
        ])->create();

        $email = new Mailable;

        $result = (new SendFormEmail)->execute($form, $email);

        Mail::assertQueued(Mailable::class);
        Mail::assertQueued(function (Mailable $mail) {
            return $mail->hasCc('test@example.com');
        });
    }

    public function test_queues_form_adds_multiple_cc()
    {
        Mail::fake();

        $form = Form::factory()->state([
            'cc' => 'test1@example.com,test2@example.com'
        ])->create();

        $email = new Mailable;

        $result = (new SendFormEmail)->execute($form, $email);

        Mail::assertQueued(Mailable::class);
        Mail::assertQueued(function (Mailable $mail) {
            return $mail->hasCc('test1@example.com') && $mail->hasCc('test2@example.com');
        });

    }

}